@@ -65,7 +65,8 @@ QTM_BEGIN_NAMESPACE
 
     Feedback effects have a duration, which is measured in
     milliseconds. Subclasses reimplement duration() to inform how long
-    the effect lasts.
+    the effect lasts.  The duration is the total time the effect will
+    last, and thus includes any envelope modifiers (attack and fade).
 
     At any given time, a feedback effect is in one of four states:
     Loading, Stopped, Running, or Paused. See the
@@ -214,7 +215,9 @@ QTM_BEGIN_NAMESPACE
     \brief duration of the feedback effect, in milliseconds.
 
     In some cases the duration will be unknown, which will be reported as 0.  If the duration
-    is infinite (via looping or other means), QFeedbackEffect::Infinite will be returned.
+    is infinite, QFeedbackEffect::Infinite will be returned.  Some subclasses may have
+    more than one type of duration (for example, \l QFeedbackHapticsEffect), and this
+    property will return the total duration of the effect.
 */
 
 /*!
@@ -303,7 +306,7 @@ bool QFeedbackEffect::supportsThemeEffect()
     it can be played:
 
     \list
-        \o duration(): The duration of the effect in milliseconds.
+        \o duration(): The total duration of the effect in milliseconds.
         \o intensity(): The intensity, e.g., how hard the device will vibrate.
     \endlist
 
@@ -347,7 +350,8 @@ bool QFeedbackEffect::supportsThemeEffect()
     \endcode
 
     When using fade-in and fade-out the total duration of the haptics
-    effect will be: attackTime() + duration() + fadeTime().
+    effect will be: duration(); the main intensity() will be played for
+    (duration() - (attackTime() + fadeTime())) milliseconds.
 
     A QFeedbackHapticsEffect is played on an
     \l{QFeedbackHapticsEffect::}{actuator()}, which is the physical component that
@@ -378,12 +382,7 @@ bool QFeedbackEffect::supportsThemeEffect()
 */
 QFeedbackHapticsEffect::QFeedbackHapticsEffect(QObject *parent) : QFeedbackEffect(parent), priv(new QFeedbackHapticsEffectPrivate)
 {
-    QList<QFeedbackActuator*> list = QFeedbackActuator::actuators();
-    if  (!list.isEmpty()) {
-        priv->actuator = list.first();
-    } else {
-        priv->actuator = new QFeedbackActuator(this);
-    }
+    setActuator(0);
 }
 
 
@@ -399,7 +398,15 @@ QFeedbackHapticsEffect::~QFeedbackHapticsEffect()
     \property QFeedbackHapticsEffect::duration
     \brief the expected duration of the effect.
 
-    This property defines the duration of the feedback effect, in milliseconds.
+    This property defines the total duration of the feedback effect, in milliseconds.
+    It includes the duration of any fade-in or fade-out parts, if any, in non-periodic
+    effects, and includes all repetitions of the period in periodic-effects, if any.
+
+    If the duration is set to a value less than attackTime() + fadeTime(), or less
+    than the period() of the effect, the waveform which will result is
+    backend-specific.
+
+    \sa fadeTime(), attackTime(), period()
 */
 int QFeedbackHapticsEffect::duration() const
 {
@@ -419,6 +426,11 @@ void QFeedbackHapticsEffect::setDuration(int msecs)
 
     This property defines the intensity of the feedback effect.
     The value can be between 0 and 1.
+
+    For non-periodic effects, the effect will be at this intensity for
+    (duration() - (attackTime() + fadeTime())) milliseconds.
+    For periodic effects, the effect will be at this intensity once per
+    period for (period() - (attackTime() + fadeTime())) milliseconds.
 */
 qreal QFeedbackHapticsEffect::intensity() const
 {
@@ -437,6 +449,14 @@ void QFeedbackHapticsEffect::setIntensity(qreal intensity)
     \brief the duration of the fade-in effect.
 
     This property defines the duration of the fade-in effect in milliseconds.
+    The effect will ramp up (or down) from attackIntensity() to intensity() in attackTime() milliseconds.
+
+    If the attack time is set to a value such that attackTime() + fadeTime()
+    is greater than duration() for non-periodic effects, or greater than
+    period() for periodic effects, the waveform which will result is
+    backend-specific.
+
+    \sa duration(), period()
 */
 int QFeedbackHapticsEffect::attackTime() const
 {
@@ -455,7 +475,8 @@ void QFeedbackHapticsEffect::setAttackTime(int msecs)
     \brief the initial intensity of the effect.
 
     This property defines the initial intensity of the effect, before it fades in.
-    It is usually lower than \l intensity.
+    It is usually lower than \l intensity.  The effect will ramp up (or down) from
+    attackIntensity() to intensity() in attackTime() milliseconds.
 */
 qreal QFeedbackHapticsEffect::attackIntensity() const
 {
@@ -474,6 +495,14 @@ void QFeedbackHapticsEffect::setAttackIntensity(qreal intensity)
     \brief the duration of the fade-out effect.
 
     This property defines the duration of the fade-out effect in milliseconds.
+    The effect will ramp down (or up) from intensity() to fadeIntensity() in fadeTime() milliseconds.
+
+    If the fade time is set to a value such that attackTime() + fadeTime()
+    is greater than duration() for non-periodic effects, or greater than
+    period() for periodic effects, the waveform which will result is
+    backend-specific.
+
+    \sa duration(), period()
 */
 int QFeedbackHapticsEffect::fadeTime() const
 {
@@ -493,6 +522,7 @@ void QFeedbackHapticsEffect::setFadeTime(int msecs)
 
     This property defines the final intensity of the effect, after it fades out.
     It is usually lower than \l intensity.
+    The effect will ramp down (or up) from intensity() to fadeIntensity() in fadeTime() milliseconds.
 */
 qreal QFeedbackHapticsEffect::fadeIntensity() const
 {
@@ -511,7 +541,8 @@ void QFeedbackHapticsEffect::setFadeIntensity(qreal intensity)
     \brief the actuator on which the effect operates.
 
     This property defines the actuator on which the effect operates.  You can only
-    change the actuator used when the effect is stopped.
+    change the actuator used when the effect is stopped.  Setting a null actuator
+    resets the effect to use the default actuator.
 */
 QFeedbackActuator* QFeedbackHapticsEffect::actuator() const
 {
@@ -524,7 +555,16 @@ void QFeedbackHapticsEffect::setActuator(QFeedbackActuator *actuator)
         return;
     }
 
-    priv->actuator = actuator;
+    if (actuator) {
+        priv->actuator = actuator;
+    } else {
+        QList<QFeedbackActuator*> list = QFeedbackActuator::actuators();
+        if  (!list.isEmpty()) {
+            priv->actuator = list.first();
+        } else {
+            priv->actuator = new QFeedbackActuator(this);
+        }
+    }
 }
 
 /*!
@@ -533,7 +573,19 @@ void QFeedbackHapticsEffect::setActuator(QFeedbackActuator *actuator)
 
     It has a default value of -1, which means that it is not a periodic effect.
     You can only change the period when the effect is stopped.
+    The duration of the effect should be set to a value larger than the
+    period of the effect if you wish the periodicity to be discernable.
     \note Not all actuators support periodic effects
+
+    The period defines the total length of the periodic envelope, which will
+    be repeated up until duration() milliseconds has elapsed.  For a periodic
+    effect, the intensity will start at attackIntensity(), ramp to intensity()
+    (where it stays for (period() - (attackTime() + fadeTime())) milliseconds),
+    then ramp to fadeIntensity().  This waveform will be repeated as many times
+    as required until the duration() has elapsed.
+
+    If the period is set to a value which is less than attackTime() + fadeTime(),
+    the waveform which will result is backend-specific.
 */
 int QFeedbackHapticsEffect::period() const
 {
